<?php

namespace CleantalkSP\Common;

class SQLSchema
{

    /**
     * @param string $name
     *
     * @return string
     */
    protected static function processSchemaName( $name ){
        return strpos( $name, 'spbc_' ) === 0
            ? substr( $name, 5 )
            : $name;
    }

    /**
     * Searches and returns schema of all types
     *
     * @param null|string $table         Name of called table
     * @return array                     Array of schemas
     * @throws \Exception                Throws if calling un-existed schema
     */
    public static function getSchema( $table = null )
    {
        $schemas__all = array_merge( static::$schemas__common, static::$schemas__blog );

        if( is_null( $table ) ) {
            return $schemas__all;
        }

        $table = static::processSchemaName( $table );

        if( array_key_exists( $table, $schemas__all ) ) {
            return array( $schemas__all[$table] );
        }

        throw new \Exception( 'Called table scheme not exist.' );
    }


    /**
     * Searches and returns schema of common type
     *
     * @param null|string $table         Name of called table
     * @return array                     Array of schemas
     * @throws \Exception                Throws if calling un-existed schema
     */
    public static function getSchemaCommon( $table = null )
    {
        if( is_null( $table ) ) {
            return static::$schemas__common;
        }

        $table = static::processSchemaName( $table );

        if( array_key_exists( $table, static::$schemas__common ) ) {
            return array( static::$schemas__common[$table] );
        }

        throw new \Exception( 'Called table scheme not exist.' );
    }

    /**
     * Searches and returns schema of blog type
     *
     * @param null|string $table         Name of called table
     * @return array                     Array of schemas
     * @throws \Exception                Throws if calling un-existed schema
     */
    public static function getSchemaBlog( $table = null )
    {
        if( is_null( $table ) ) {
            return static::$schemas__blog;
        }

        $table = static::processSchemaName( $table );

        if( array_key_exists( $table, static::$schemas__blog ) ) {
            return array( static::$schemas__blog[$table] );
        }

        throw new \Exception( 'Called table scheme not exist.' );
    }

}