<?php


namespace CleantalkSP\SpbctWP;


class SQLSchema extends \CleantalkSP\Common\SQLSchema {
    
    /**
     * Set of SQL-schemas for tables in array
     * Set for all websites. Should installed with a main database prefix
     *
     * @var array
     */
    protected static $schemas__common = array(

        'firewall_data' => 'CREATE TABLE IF NOT EXISTS %sspbc_firewall_data (
			`id` char(32) NOT NULL,
			`network1` INT(10) UNSIGNED NOT NULL DEFAULT "0",
			`network2` INT(10) UNSIGNED NOT NULL DEFAULT "0",
			`network3` INT(10) UNSIGNED NOT NULL DEFAULT "0",
			`network4` INT(10) UNSIGNED NOT NULL DEFAULT "0",
			`mask1` INT(10) UNSIGNED NOT NULL DEFAULT "0",
			`mask2` INT(10) UNSIGNED NOT NULL DEFAULT "0",
			`mask3` INT(10) UNSIGNED NOT NULL DEFAULT "0",
			`mask4` INT(10) UNSIGNED NOT NULL DEFAULT "0",
			`country_code` char(2) NULL DEFAULT NULL,
			`status` TINYINT(2) NULL,
			INDEX `network` (`network1`, `network2`, `network3`, `network4`, `mask1`, `mask2`, `mask3`, `mask4`),
			PRIMARY KEY (`id`)
		);',

        'scan_results' => 'CREATE TABLE IF NOT EXISTS %sspbc_scan_results (
            `path` VARCHAR(1024) NOT NULL,
            `size` INT(10) NOT NULL,
            `perms` INT(4) NOT NULL,
            `mtime` INT(11) NOT NULL,
            `source_type` ENUM("CORE", "PLUGIN", "THEME") NULL DEFAULT NULL,
            `source` VARCHAR(300) NULL DEFAULT NULL,
            `source_status` SET("UP_TO_DATE","OUTDATED","NOT_IN_DIRECTORY","UNKNOWN") NULL DEFAULT NULL,
            `version` VARCHAR(20) NULL DEFAULT NULL,
            `checked` ENUM("NO", "YES", "YES_SIGNATURE", "YES_HEURISTIC") NOT NULL DEFAULT "NO",
            `status` ENUM("UNKNOWN","OK","APROVED","MODIFIED","INFECTED","QUARANTINED") NOT NULL DEFAULT "UNKNOWN",
            `severity` ENUM("CRITICAL", "DANGER", "SUSPICIOUS", "NONE") NULL DEFAULT NULL,
            `weak_spots` VARCHAR(2048) NULL DEFAULT NULL,
            `difference` VARCHAR(1024) NULL DEFAULT NULL,
            `last_sent` INT(11) NULL DEFAULT NULL,
            `fast_hash` VARCHAR(32) NULL DEFAULT NULL,
            `full_hash` VARCHAR(32) NULL DEFAULT NULL,
            `real_full_hash` VARCHAR(32) NULL DEFAULT NULL,
            `previous_state` VARCHAR(1024) NULL DEFAULT NULL,
            `q_path` VARCHAR(1024) NULL DEFAULT NULL,
            `q_time` INT(11) NULL DEFAULT NULL,
            UNIQUE (`fast_hash`)
        );',

        'scan_links_logs' => 'CREATE TABLE IF NOT EXISTS %sspbc_scan_links_logs (
            `link_id` INT(11) NOT NULL AUTO_INCREMENT,
            `scan_id` INT(11) NOT NULL,
            `domain` TINYTEXT NOT NULL,
            `link` VARCHAR(2048) NOT NULL,
            `link_text` VARCHAR(2048) NOT NULL,
            `page_url` VARCHAR(2048) NOT NULL,
            `spam_active` TINYINT(1) NULL,
            PRIMARY KEY (`link_id`),
            INDEX `spam_active` (`spam_active`),
            INDEX `scan_id` (`scan_id`),
            INDEX `domain` (`domain`(40))
            );',

        'scan_frontend' => 'CREATE TABLE IF NOT EXISTS %sspbc_scan_frontend (
            `page_id` VARCHAR(1024) NOT NULL,
            `url` VARCHAR(1024) NOT NULL,
            `dbd_found` TINYINT NULL,
            `redirect_found` TINYINT NULL,
            `signature` TINYINT NULL,
            `bad_code` TINYINT NULL,
            `weak_spots` VARCHAR(1024) NULL
            );',

        'scan_signatures' => 'CREATE TABLE IF NOT EXISTS %sspbc_scan_signatures (
            `id` INT(11) UNSIGNED NOT NULL AUTO_INCREMENT,
            `name` VARCHAR(128) NOT NULL,
            `body` VARCHAR(512) NOT NULL,
            `type` ENUM("FILE","CODE_PHP","CODE_HTML","CODE_JS","WAF_RULE") NOT NULL,
            `attack_type` SET("SQL_INJECTION","XSS","MALWARE","EXPLOIT") NOT NULL,
            `submitted` DATETIME NOT NULL,
            `cci` TEXT NULL DEFAULT NULL,
            PRIMARY KEY (`id`),
            UNIQUE KEY (`name`)
            );',

        'backuped_files' => 'CREATE TABLE IF NOT EXISTS %sspbc_backuped_files (
            `id` INT(11) UNSIGNED NOT NULL AUTO_INCREMENT,
            `backup_id` INT(11) UNSIGNED NOT NULL,
            `real_path` VARCHAR(512) NOT NULL,
            `back_path` VARCHAR(512) NOT NULL,
            PRIMARY KEY (`id`)
            );',

        'backups' => 'CREATE TABLE IF NOT EXISTS %sspbc_backups (
            `backup_id` INT(11) UNSIGNED NOT NULL AUTO_INCREMENT,
            `type` ENUM("FILE","ALL","SIGNATURES") NOT NULL DEFAULT "FILE",
            `datetime` DATETIME NOT NULL,
            `status` ENUM("PROCESSING", "BACKUPED", "ROLLBACK", "ROLLBACKED", "STOPPED") NOT NULL DEFAULT "PROCESSING",
            PRIMARY KEY (`backup_id`) 
            );',
        
        'alt_sessions' => 'CREATE TABLE IF NOT EXISTS `%sspbc_sessions` (
            `id` VARCHAR(64) NOT NULL,
            `name` VARCHAR(40) NOT NULL,
            `value` TEXT NULL DEFAULT NULL,
            `last_update` DATETIME NULL DEFAULT NULL,
            PRIMARY KEY (`name`(40), `id`(64)))',
    );
    
    /**
     * Set of SQL-schemas for tables in array
     * Set for a blog only. Should installed with a blog database prefix
     * 
     * @var array
     */
    protected static $schemas__blog = array(
        
        'monitoring_users' => 'CREATE TABLE IF NOT EXISTS %sspbc_monitoring_users (
            `user_id` int(11) NOT NULL,
            `user_login` varchar(60) NOT NULL,
            `last_activity` INT(15) NOT NULL,
            `page` VARCHAR(500) NULL,
            `ip` VARCHAR(50) DEFAULT NULL,
            `role` varchar(64) DEFAULT NULL,
            `user_agent` VARCHAR(1024) DEFAULT NULL,
            PRIMARY KEY (`user_id`),
            KEY `timestamp` (`last_activity`)
            ) DEFAULT CHARSET=latin1;',
        
        'auth_logs' => 'CREATE TABLE IF NOT EXISTS %sspbc_auth_logs (
            `id` int(11) NOT NULL AUTO_INCREMENT,
            `datetime` datetime NOT NULL,
            `timestamp_gmt` INT(10) NOT NULL,
            `user_login` varchar(60) NOT NULL,
            `event` varchar(32) NOT NULL,
            `page` VARCHAR(500) NULL,
            `page_time` VARCHAR(10) NULL,
            `blog_id` int(11) NOT NULL,
            `auth_ip` VARCHAR(50) DEFAULT NULL,
            `role` varchar(64) DEFAULT NULL,
            `user_agent` VARCHAR(1024) DEFAULT NULL,
            `browser_sign` VARCHAR(32) DEFAULT NULL,
            PRIMARY KEY (`id`),
            KEY `datetime` (`datetime`,`event`)
            ) DEFAULT CHARSET=latin1 AUTO_INCREMENT=1;',
        
        'firewall__personal_ips' => 'CREATE TABLE IF NOT EXISTS %sspbc_firewall__personal_ips (
		    `id` INT(10) NOT NULL AUTO_INCREMENT,
			`network1` INT(10) UNSIGNED NOT NULL DEFAULT "0",
			`network2` INT(10) UNSIGNED NOT NULL DEFAULT "0",
			`network3` INT(10) UNSIGNED NOT NULL DEFAULT "0",
			`network4` INT(10) UNSIGNED NOT NULL DEFAULT "0",
			`mask1` INT(10) UNSIGNED NOT NULL DEFAULT "0",
			`mask2` INT(10) UNSIGNED NOT NULL DEFAULT "0",
			`mask3` INT(10) UNSIGNED NOT NULL DEFAULT "0",
			`mask4` INT(10) UNSIGNED NOT NULL DEFAULT "0",
			`status` TINYINT(3) NOT NULL DEFAULT "0",
			PRIMARY KEY (`id`),
			INDEX `network` (`network1`, `network2`, `network3`, `network4`, `mask1`, `mask2`, `mask3`, `mask4`)
		)',
        
        'firewall__personal_countries' => 'CREATE TABLE IF NOT EXISTS %sspbc_firewall__personal_countries (
		    `id` INT(11) NOT NULL AUTO_INCREMENT,
		    `country_code` CHAR(2) NOT NULL,
		    `status` TINYINT(2) NOT NULL,
		    PRIMARY KEY (`id`)
		)',
        
        'firewall_logs' => "CREATE TABLE IF NOT EXISTS %sspbc_firewall_logs (
            `entry_id` VARCHAR(40) NOT NULL,
            `ip_entry` VARCHAR(50) NULL,
            `status` ENUM('PASS','PASS_BY_TRUSTED_NETWORK','PASS_BY_WHITELIST','DENY','DENY_BY_NETWORK','DENY_BY_DOS','DENY_BY_WAF_XSS','DENY_BY_WAF_SQL','DENY_BY_WAF_FILE','DENY_BY_WAF_EXPLOIT','DENY_BY_SEC_FW','DENY_BY_SPAM_FW','DENY_BY_BFP') NULL DEFAULT NULL,
            `pattern` VARCHAR(1024) NULL,
            `requests` INT NULL,
            `page_url` VARCHAR(1024) NULL,
            `request_method` VARCHAR(5) NULL,
            `x_forwarded_for` VARCHAR(15) NULL,
            `http_user_agent` VARCHAR(300) NULL,
            `network` INT(10) UNSIGNED NULL DEFAULT NULL,
            `mask` INT(10) UNSIGNED NULL DEFAULT NULL,
            `country_code` CHAR(2) NULL DEFAULT NULL,
            `is_personal` TINYINT(2) UNSIGNED NULL DEFAULT NULL,
            `entry_timestamp` INT NOT NULL ,
            PRIMARY KEY (`entry_id`)
		);",
        
        'traffic_control_logs' => "CREATE TABLE IF NOT EXISTS %sspbc_traffic_control_logs (
			`id` VARCHAR(32) NOT NULL,
			`log_type` TINYINT NULL DEFAULT NULL,
			`ip` VARCHAR(40) NOT NULL,
			`entries` INT DEFAULT 0,
			`interval_start` INT NOT NULL,
			PRIMARY KEY (`id`),
			INDEX `bfp_index` (`interval_start`, `log_type`)
		);",
        
        'bfp_blocked' => "CREATE TABLE IF NOT EXISTS %sspbc_bfp_blocked (
			`id` VARCHAR(32) NOT NULL,
			`ip` VARCHAR(15) NOT NULL,
			`start_time_of_blocking` INT NOT NULL,
			PRIMARY KEY (`id`)
		);"
    );  
    
    
    
}